<?php
/**
 * DSPAM training plugin
 *
 * Author: Holger de Carne <hdecarne@gmail.com>
 *
 */

class Plugindspam extends Plugin {

	private $_log;

	function Plugindspam() {
		$defaults = $this->getDefaults();
		$this->_log = $defaults['dspam_log'];
		$this->log('Status: Plugindspam constructed');
	}

	function init() {
		$this->registerHook('server.module.maillistmodule.execute.before');
		$this->log('Status: Plugindspam initialized');
	}

	function execute($eventID, &$data) {
		switch($eventID) {
		case 'server.module.maillistmodule.execute.before':
			$this->log('Event: server.module.maillistmodule.execute.before');
			$this->mailListModuleExecute($data['moduleObject']);
			break;
		}
	}

	function mailListModuleExecute(&$maillist) {
		foreach($maillist->data as $action) {
			if(isset($action['attributes']) && isset($action['attributes']['type'])) {
				$store = $maillist->getActionStore($action);
				$parentEntryID = $maillist->getActionParentEntryID($action);
				$entryID = $maillist->getActionEntryID($action);
				$actionType = $action['attributes']['type'];
				$this->log('Action: '.$actionType);
				switch($actionType) {
				case 'dspamTrainSPAM':
					$result = $this->invokeDSPAMTrain($store, $entryID, '--source=error --class=spam');
					if ($result) {
						$maillist->delete($store, $parentEntryID, array($entryID), $action);
					}
					break;
				case 'dspamTrainHAM':
					$result = $this->invokeDSPAMTrain($store, $entryID, '--source=error --class=innocent');
					break;
				}
			}
		}
	}

	function invokeDSPAMTrain($store, $entryID, $cmdLine) {
		$status = false;
		$defaults = $this->getDefaults();
		$session = $this->getSession();
		if(isset($defaults['dspam_cmd'])) {
			$message = mapi_msgstore_openentry($store, $entryID);
			$props = mapi_getprops($message, array(PR_TRANSPORT_MESSAGE_HEADERS));
			$addressBook = mapi_openaddressbook($session);
			$stream = mapi_inetmapi_imtoinet($session, $addressBook, $message, array());
			if($stream) {
				if(isset($defaults['dspam_user']) && $defaults['dspam_user']) {
					$user = $defaults['dspam_user'];
				} else {
					$user = $session->getUserName();
				}
				$cmd = $defaults['dspam_cmd'].' --user '.$user.' '.$cmdLine;
				$this->log('Debug: Invoking command "'.$cmd.'"');
				$pipe = popen($cmd, 'w');
				if($pipe) {
					$this->log('Debug: Streaming message to command');
					fwrite($pipe, $props[PR_TRANSPORT_MESSAGE_HEADERS]);
					fwrite($pipe, "\n");
					$this->log($props[PR_TRANSPORT_MESSAGE_HEADERS]);
					while(($read = mapi_stream_read($stream, 4096))) {
						fwrite($pipe, $read);
						$this->log($read, false);
					}
					$this->log('');
					$pclose = pclose($pipe);
					$status = ($pclose == 0);
					if(!$status) {
						$this->log('Error: Command "'.$cmd.'" finished with status '.$pclose);
					} else {
						$this->log('Debug: Command "'.$cmd.'" finished with status '.$pclose);
					}
				} else {
					$this->log('Error: Failed to open command "'.$cmd.'"');
				}
			} else {
				$this->log('Error: Failed to open MAPI stream');
			}
		} else {
			$this->log('Error: Parameter dspam_cmd is not defined in config.php');
		}
		return $status;
	}

	function getDefaults() {
		return $GLOBALS['pluginconfig']['dspam']['defaults'];
	}

	function getSession() {
		return $GLOBALS["mapisession"]->getSession();
	}

	function log($message, $newline = true) {
		if($this->_log) {
			$log = fopen($this->_log, 'a');
			if($log) {
				fwrite($log, $message);
				if($newline) {
					fwrite($log, "\n");
				}
				fclose($log);
			}
		}
	}

}

?>
